// ------------------------------------------------------------------------------
// Line - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Line.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

// Initiate

int num_s;
int num_o;
int num_p;

float fb = 0.6;             // Bounce constant.
float fk = 0.1;             // Friction constant.
float fr;                   // Spring constant.
float fg;                   // Gravity constant.
float fd;                   // Damping constant.
float fl;                   // Length constant.

Spring[] Springs;           // Create spring array.
Object[] Objects;           // Create object array.  
HScroller LengthS;          // Create scrollers.
HScroller GravityS;         // Create scrollers.
HScroller SpringS;          // Create scrollers.
HScroller DampingS;         // Create scrollers.

PFont font; 

void setup()
{
  // Applet Settings
  
  size(800,420);
  framerate(30);
  smooth();
  font = loadFont("Verdana-10.vlw"); 
  textFont(font, 10); 

  // Create Scrollers
  
  LengthS = new HScroller(0.5, 3, 0.5, 700, 310, 50);
  GravityS = new HScroller(0, 3, 1, 700, 330, 50);
  SpringS = new HScroller(0.0, 2, 2, 700, 350, 50);
  DampingS = new HScroller(0.1, 0.85, 0.80, 700, 370, 50);
    
  // Create Objects
  
  num_o = 20;         
  Objects = new Object[num_o];
  
  Objects[0] = new Object(200, 160, 2, 5, 0);
  Objects[1] = new Object(225, 210, 2, 5, 1);
  Objects[2] = new Object(250, 210, 2, 5, 1);
  Objects[3] = new Object(275, 210, 2, 5, 1);
  Objects[4] = new Object(300, 210, 2, 5, 1);
  Objects[5] = new Object(325, 210, 2, 5, 1);
  Objects[6] = new Object(350, 210, 2, 5, 1);
  Objects[7] = new Object(375, 210, 2, 5, 1);
  Objects[8] = new Object(400, 210, 2, 5, 1);
  Objects[9] = new Object(425, 210, 2, 5, 1);
  Objects[10] = new Object(450, 210, 2, 5, 1);
  Objects[11] = new Object(475, 210, 2, 5, 1);
  Objects[12] = new Object(500, 210, 2, 5, 1);
  Objects[13] = new Object(525, 210, 2, 5, 1);
  Objects[14] = new Object(550, 210, 2, 5, 1);
  Objects[15] = new Object(575, 210, 2, 5, 1);
  Objects[16] = new Object(600, 210, 2, 5, 1);
  Objects[17] = new Object(600, 160, 2, 5, 0);
  Objects[18] = new Object(200, 310, 6, 75, 1);
  Objects[19] = new Object(600, 310, 4, 50, 1);
  
  // Create springs
  
  num_s = 17;
  Springs = new Spring[num_s];

  Springs[0] = new Spring(0,1,25);
  Springs[1] = new Spring(1,2,25);
  Springs[2] = new Spring(2,3,25);
  Springs[3] = new Spring(3,4,25);
  Springs[4] = new Spring(4,5,25);
  Springs[5] = new Spring(5,6,25);
  Springs[6] = new Spring(6,7,25);
  Springs[7] = new Spring(7,8,25);
  Springs[8] = new Spring(8,9,25);
  Springs[9] = new Spring(9,10,25);
  Springs[10] = new Spring(10,11,25);
  Springs[11] = new Spring(11,12,25);
  Springs[12] = new Spring(12,13,25);
  Springs[13] = new Spring(13,14,25);
  Springs[14] = new Spring(14,15,25);
  Springs[15] = new Spring(15,16,25);
  Springs[16] = new Spring(16,17,25);  
}

void draw()
{
  background(255,255,255);
  
  for ( int i=0; i<num_s; i++ )
  {
    Springs[i].pull();
  }
   
  for ( int i=0; i<num_o; i++ )
  {
    Objects[i].drag();
    Objects[i].bounce();
    Objects[i].collide();
    Objects[i].fall();
    Objects[i].move();
  }
  
  for ( int i=0; i<num_s; i++ )
  {
    Springs[i].display();
  }
  
  for ( int i=0; i<num_o; i++ )
  {
    Objects[i].display();
  }
  
  fl = LengthS.update();
  fg = GravityS.update();
  fr = SpringS.update();
  fd = DampingS.update();
  
  fill(100,100,100);
  text("l", 760,312);
  text("g", 760,332);
  text("k", 760,352);
  text("b", 760,372);
  text("Shift + Click", 700, 392);
}

void mousePressed()
{
  for ( int i=0; i<num_o; i++ )
  {
    Objects[i].checkClick();
  }
  
  LengthS.checkClick();
  GravityS.checkClick();
  SpringS.checkClick();
  DampingS.checkClick();
}
