// ------------------------------------------------------------------------------
// Line - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Object.java
// The things that get connected to other objects via strings
// ------------------------------------------------------------------------------

class Object
{
  float x;
  float y;
  float m;
  float r;
  float f;
  float wx;
  float wy;
  float tx;
  float ty;
  float bx;
  float by;
  float vx;
  float vy;
  boolean locked;
  
  Object( float X, float Y, float M, float R, float F )
  {
    x = abs(X);
    y = abs(Y);
    m = M;
    r = R;
    f = F;
    
    tx = ty = vx = vy = 0;
    locked = false;
  }
  
  void display()
  {
    stroke(0,0,0);
    strokeWeight(3);
    fill(100+f*155, 100+f*155, 100+f*155);
    ellipse(x,y,2*r,2*r);
  }
  
  void bounce()
  {
    if ( y + vy + r > height-4 )
    {
      y = height - r - 4;
      vx *= fk;
      vy *= -fb;
    }
    if ( y + vy - r < 4 )
    {
      y = r + 4;
      vx *= fk;
      vy *= -fb;
    }
    if ( x + vx + r > width-4 )
    {
      x = width - r - 4;
      vx *= -fb;
      vy *= fk;
    }
    if ( x + vx - r < 4 )
    {
      x = r + 4;
      vx *= -fb;
      vy *= fk;
    }
  }
  
  void fall()
  {
    wx = 0;
    wy = m*fg;
  }
  
  void collide()
  {
    for ( int i=0; i<num_o; i++ )
    {
      float X = Objects[i].x;
      float Y = Objects[i].y;
      float R = Objects[i].r;
      float M = Objects[i].m;

      float deltax = X-x;
      float deltay = Y-y;
      float d = sqrt(pow(deltax,2)+pow(deltay,2));
      
      if ( d < r + R + 6 && d > 0 )
      {
        // We add the 6 to compensate for the
        // stroke thickness the balls have
        float dD = r + R + 6 - d;
        float theta = atan2(deltay,deltax);
        
        bx += -dD*cos(theta)*M/(m+M);
        by += -dD*sin(theta)*M/(m+M);
      }
    }
  }

  void move()
  {
    float fx = tx + wx + bx;
    float fy = ty + wy + by;
    
    bx = by = tx = ty = 0;
    
    float ax = fx/m;
    float ay = fy/m;
    
    vx += f*ax;
    vy += f*ay;
    
    vx *= fd;
    vy *= fd;
       
    x += vx;
    y += vy;
  }
  
  void checkClick()
  {
    if ( over() )
    {
      locked = true;
      if ( keyPressed )
      {
        if ( keyCode == SHIFT )
        {
          if ( f == 0 ) { f = 1; }
          else if ( f == 1 ) { f = 0; }
        }
      }
    }
  }
   
  void drag()
  {
    if ( !mousePressed )
    {
      locked = false;
    }
    if ( locked )
    {
      assignPos(mouseX, mouseY);
    }
  }
  
  void assignPos( float X, float Y )
  {
    x = X;
    y = Y;
    
    tx = ty = wx = wy = vx = vy = 0;
  }
  
  boolean over() 
  { 
    if( sqrt(pow(mouseX - x,2)+pow(mouseY-y,2)) < r ) 
    { 
      return true; 
    } 
    else
    { 
      return false; 
    } 
  }
}
